// src/use-isomorphic-effect.ts
import { useLayoutEffect, useEffect } from "react";
var useIsomorphicEffect = typeof window === "undefined" ? useEffect : useLayoutEffect;

// src/use-previous.ts
import { useEffect as useEffect2, useRef } from "react";
function usePrevious(state) {
  const ref = useRef();
  useEffect2(() => {
    ref.current = state;
  });
  return ref.current;
}

// src/use-previous-distinct.ts
import { useRef as useRef2 } from "react";
function usePreviousDistinct(value, compare = (prev, next) => prev === next) {
  const mountedRef = useRef2(false);
  const previousRef = useRef2();
  const currentRef = useRef2(value);
  if (mountedRef.current && !compare(currentRef.current, value)) {
    previousRef.current = currentRef.current;
    currentRef.current = value;
  }
  mountedRef.current = true;
  return previousRef.current;
}

// src/use-business.ts
import { useMemo, useSyncExternalStore, useCallback, useRef as useRef3 } from "react";
import { deepEqual } from "fast-equals";
var SEPARATOR = "__@.@__";
function useBusiness(emitter, events, evalFn, args) {
  const eventStr = events.join(SEPARATOR);
  const subscribe = useCallback(
    (listener) => {
      const types = eventStr.split(SEPARATOR);
      types.forEach((type) => {
        emitter.on(type, listener);
      });
      return () => {
        types.forEach((type) => {
          emitter.off(type, listener);
        });
      };
    },
    [emitter, eventStr]
  );
  const previousArgs = usePreviousDistinct(args, deepEqual);
  const currentArgs = useMemo(
    () => previousArgs && deepEqual(args, previousArgs) ? previousArgs : args,
    [args, previousArgs]
  );
  const evalFnRef = useRef3(evalFn);
  evalFnRef.current = evalFn;
  const currentValueRef = useRef3();
  const getSnapshot = useCallback(() => {
    const nextValue = evalFnRef.current(emitter, currentArgs);
    const currentValue = currentValueRef.current;
    currentValueRef.current = deepEqual(nextValue, currentValue) ? currentValue : nextValue;
    return currentValueRef.current;
  }, [emitter, currentArgs]);
  return useSyncExternalStore(subscribe, getSnapshot);
}
export {
  useBusiness,
  useIsomorphicEffect,
  usePrevious,
  usePreviousDistinct
};
